@@ -327,8 +327,68 @@ class LibvirtDriverTestCase(base.BaseTestCase):
 
         self.assertIn(expected, conn_mock.defineXML.call_args[0][0])
 
+    @mock.patch('libvirt.open', autospec=True)
+    def test_set_boot_device_network_from_hd(self, libvirt_mock):
+        with open('sushy_tools/tests/unit/emulator/'
+                  'domain_to_boot_pxe.xml', 'r') as f:
+            data = f.read()
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        with mock.patch.object(
+                self.test_driver, 'get_power_state', return_value='Off'):
+            self.test_driver.set_boot_device(self.uuid, 'Pxe')
+
+        conn_mock.defineXML.assert_called_once_with(mock.ANY)
+
+        newtree = ET.fromstring(conn_mock.defineXML.call_args[0][0])
+        # check that os section does not have boot defined.
+        # We find all os sections if any. Then count about of boot sections.
+        # And the final summ should be 0
+        os_boot_amount = sum(
+            [len(ossec.findall('boot')) for ossec in newtree.findall('os')])
+        self.assertEqual(0, os_boot_amount)
+
+        # check that Network device has order=1
+        interface_orders = [
+            theint.find('boot').get('order')
+            for theint in newtree.find('devices').findall('interface')]
+        self.assertIn('1', interface_orders)
+
+        # Check that we have at least one hd device set after a network device
+        diskdrives_order_sum = len([
+            thedrive.find('boot').get('order')
+            for thedrive in newtree.find('devices').findall('disk')])
+        self.assertEqual(2, diskdrives_order_sum)
+
+        # Check overal config to match expected fixture
+        expected = '<domain type="qemu">\n  <name>QEmu-fedora-i686</name>\n '\
+            ' <uuid>c7a5fdbd-cdaf-9455-926a-d65c16db1809</uuid>\n  '\
+            '<memory>219200</memory>\n  '\
+            '<currentMemory>219200</currentMemory>\n  <vcpu>2</vcpu>\n  '\
+            '<os>\n    <type arch="x86_64" machine="pc">hvm</type>\n    '\
+            '<loader type="rom" />\n  </os>\n  <devices>\n    '\
+            '<emulator>/usr/bin/qemu-system-x86_64</emulator>\n    '\
+            '<disk type="file" device="cdrom">\n      '\
+            '<source file="/home/user/boot.iso" />\n      '\
+            '<target dev="hdc" />\n      <readonly />\n    '\
+            '<boot order="3" /></disk>\n    '\
+            '<disk type="file" device="disk">\n      '\
+            '<source file="/home/user/fedora.img" />\n      '\
+            '<target dev="hda" />\n    <boot order="2" /></disk>\n    '\
+            '<interface type="network">\n      '\
+            '<source network="default" />\n      '\
+            '<mac address="52:54:00:da:ac:54" />\n      '\
+            '<model type="virtio" />\n      <address type="pci" '\
+            'domain="0x0000" bus="0x01" slot="0x01" function="0x0" />\n    '\
+            '<boot order="1" /></interface>\n    '\
+            '<graphics type="vnc" port="-1" />\n  </devices>\n</domain>'
+        self.assertIn(expected, conn_mock.defineXML.call_args[0][0])
+
     @mock.patch('libvirt.openReadOnly', autospec=True)
-    def test_get_boot_mode(self, libvirt_mock):
+    def test_get_boot_mode_legacy(self, libvirt_mock):
         with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
             data = f.read()
 
@@ -340,13 +400,27 @@ class LibvirtDriverTestCase(base.BaseTestCase):
 
         self.assertEqual('Legacy', boot_mode)
 
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_get_boot_mode_uefi(self, libvirt_mock):
+        with open('sushy_tools/tests/unit/emulator/domain-q35_uefi.xml',
+                  'r') as f:
+            data = f.read()
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        boot_mode = self.test_driver.get_boot_mode(self.uuid)
+
+        self.assertEqual('UEFI', boot_mode)
+
     @mock.patch('libvirt.open', autospec=True)
     @mock.patch('libvirt.openReadOnly', autospec=True)
     def test_set_boot_mode(self, libvirt_mock, libvirt_rw_mock):
         with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
             data = f.read()
 
-        conn_mock = libvirt_mock.return_value
+        conn_mock = libvirt_rw_mock.return_value
         domain_mock = conn_mock.lookupByUUID.return_value
         domain_mock.XMLDesc.return_value = data
 
@@ -359,47 +433,26 @@ class LibvirtDriverTestCase(base.BaseTestCase):
 
     @mock.patch('libvirt.open', autospec=True)
     @mock.patch('libvirt.openReadOnly', autospec=True)
-    def test_set_boot_mode_known_loader(self, libvirt_mock, libvirt_rw_mock):
-        with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
+    def test_set_boot_mode_legacy(self, libvirt_mock, libvirt_rw_mock):
+        with open('sushy_tools/tests/unit/emulator/domain-q35_uefi.xml',
+                  'r') as f:
             data = f.read()
 
-        conn_mock = libvirt_mock.return_value
+        conn_mock = libvirt_rw_mock.return_value
         domain_mock = conn_mock.lookupByUUID.return_value
         domain_mock.XMLDesc.return_value = data
 
         with mock.patch.object(
                 self.test_driver, 'get_power_state', return_value='Off'):
-            self.test_driver.set_boot_mode(self.uuid, 'UEFI')
+            self.test_driver.set_boot_mode(self.uuid, 'Legacy')
 
         conn_mock = libvirt_rw_mock.return_value
         xml_document = conn_mock.defineXML.call_args[0][0]
         tree = ET.fromstring(xml_document)
         os_element = tree.find('os')
         loader_element = os_element.find('loader')
-        self.assertEqual(
-            'pflash', loader_element.get('type'))
-        self.assertEqual(
-            '/usr/share/OVMF/OVMF_CODE.fd',
-            loader_element.text)
-
-    @mock.patch('libvirt.open', autospec=True)
-    @mock.patch('libvirt.openReadOnly', autospec=True)
-    def test_set_boot_mode_unknown_loader_path(
-            self, libvirt_mock, libvirt_rw_mock):
-        with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
-            data = f.read()
-
-        conn_mock = libvirt_mock.return_value
-        domain_mock = conn_mock.lookupByUUID.return_value
-        domain_mock.XMLDesc.return_value = data
-
-        with mock.patch.dict(
-                self.test_driver.KNOWN_BOOT_LOADERS, {}, clear=True):
-            with mock.patch.object(
-                    self.test_driver, 'get_power_state', return_value='Off'):
-                self.assertRaises(
-                    error.FishyError, self.test_driver.set_boot_mode,
-                    self.uuid, 'Uefi')
+        self.assertIsNone(loader_element.text)
+        self.assertNotIn('readonly', loader_element.attrib)
 
     @mock.patch('libvirt.open', autospec=True)
     @mock.patch('libvirt.openReadOnly', autospec=True)
@@ -413,7 +466,7 @@ class LibvirtDriverTestCase(base.BaseTestCase):
 
         data = ET.tostring(tree)
 
-        conn_mock = libvirt_mock.return_value
+        conn_mock = libvirt_rw_mock.return_value
         domain_mock = conn_mock.lookupByUUID.return_value
         domain_mock.XMLDesc.return_value = data
 
@@ -435,7 +488,7 @@ class LibvirtDriverTestCase(base.BaseTestCase):
 
         data = ET.tostring(tree)
 
-        conn_mock = libvirt_mock.return_value
+        conn_mock = libvirt_rw_mock.return_value
         domain_mock = conn_mock.lookupByUUID.return_value
         domain_mock.XMLDesc.return_value = data
 
@@ -457,7 +510,7 @@ class LibvirtDriverTestCase(base.BaseTestCase):
 
         data = ET.tostring(tree)
 
-        conn_mock = libvirt_mock.return_value
+        conn_mock = libvirt_rw_mock.return_value
         domain_mock = conn_mock.lookupByUUID.return_value
         domain_mock.XMLDesc.return_value = data
 
@@ -467,34 +520,6 @@ class LibvirtDriverTestCase(base.BaseTestCase):
                 error.FishyError, self.test_driver.set_boot_mode,
                 self.uuid, 'Uefi')
 
-    @mock.patch('libvirt.open', autospec=True)
-    @mock.patch('libvirt.openReadOnly', autospec=True)
-    def test_set_boot_mode_no_type(self, libvirt_mock, libvirt_rw_mock):
-        with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
-            data = f.read()
-
-        tree = ET.fromstring(data)
-        os_element = tree.find('os')
-        type_element = os_element.find('type')
-        os_element.remove(type_element)
-
-        data = ET.tostring(tree)
-
-        conn_mock = libvirt_mock.return_value
-        domain_mock = conn_mock.lookupByUUID.return_value
-        domain_mock.XMLDesc.return_value = data
-
-        with mock.patch.object(
-                self.test_driver, 'get_power_state', return_value='Off'):
-            self.test_driver.set_boot_mode(self.uuid, 'UEFI')
-
-        conn_mock = libvirt_rw_mock.return_value
-        xml_document = conn_mock.defineXML.call_args[0][0]
-        tree = ET.fromstring(xml_document)
-        os_element = tree.find('os')
-        # NOTE(etingof): should enforce default loader
-        self.assertIsNone(os_element.find('loader'))
-
     @mock.patch('libvirt.openReadOnly', autospec=True)
     def test_get_boot_image(self, libvirt_mock):
         with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
@@ -1117,3 +1142,106 @@ class LibvirtDriverTestCase(base.BaseTestCase):
 
         self.test_driver.find_or_create_storage_volume(vol_data)
         pool_mock.createXML.assert_called_once_with(mock.ANY)
+
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_get_secure_boot_off(self, libvirt_mock):
+        with open('sushy_tools/tests/unit/emulator/domain-q35_uefi.xml',
+                  'r') as f:
+            data = f.read()
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.assertFalse(self.test_driver.get_secure_boot(self.uuid))
+
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_get_secure_boot_on(self, libvirt_mock):
+        with open('sushy_tools/tests/unit/emulator/domain-q35_uefi_secure.xml',
+                  'r') as f:
+            data = f.read()
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.assertTrue(self.test_driver.get_secure_boot(self.uuid))
+
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_get_secure_boot_not_uefi(self, libvirt_mock):
+        with open('sushy_tools/tests/unit/emulator/domain-q35.xml', 'r') as f:
+            data = f.read()
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.assertRaises(error.NotSupportedError,
+                          self.test_driver.get_secure_boot, self.uuid)
+
+    @mock.patch('libvirt.open', autospec=True)
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_set_secure_boot_on(self, libvirt_mock, libvirt_rw_mock):
+        with open('sushy_tools/tests/unit/emulator/domain-q35_uefi.xml',
+                  'r') as f:
+            data = f.read()
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.test_driver.set_secure_boot(self.uuid, True)
+
+        conn_mock = libvirt_rw_mock.return_value
+        xml_document = conn_mock.defineXML.call_args[0][0]
+        tree = ET.fromstring(xml_document)
+        os_element = tree.find('os')
+        loader_element = os_element.find('loader')
+        nvram_element = os_element.find('nvram')
+        self.assertEqual('yes', loader_element.get('secure'))
+        self.assertEqual('/usr/share/OVMF/OVMF_CODE.secboot.fd',
+                         loader_element.text)
+        self.assertEqual('/usr/share/OVMF/OVMF_VARS.secboot.fd',
+                         nvram_element.get('template'))
+        self.assertEqual('/var/lib/libvirt/nvram-%s.secboot.fd' %
+                         self.uuid, nvram_element.text)
+
+    @mock.patch('libvirt.open', autospec=True)
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_set_secure_boot_off(self, libvirt_mock, libvirt_rw_mock):
+        with open('sushy_tools/tests/unit/emulator/domain-q35_uefi_secure.xml',
+                  'r') as f:
+            data = f.read()
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.test_driver.set_secure_boot(self.uuid, False)
+
+        conn_mock = libvirt_rw_mock.return_value
+        xml_document = conn_mock.defineXML.call_args[0][0]
+        tree = ET.fromstring(xml_document)
+        os_element = tree.find('os')
+        loader_element = os_element.find('loader')
+        nvram_element = os_element.find('nvram')
+        self.assertEqual('no', loader_element.get('secure'))
+        self.assertEqual('/usr/share/OVMF/OVMF_CODE.secboot.fd',
+                         loader_element.text)
+        self.assertEqual('/usr/share/OVMF/OVMF_VARS.fd',
+                         nvram_element.get('template'))
+        self.assertEqual('/var/lib/libvirt/nvram-%s.fd' %
+                         self.uuid, nvram_element.text)
+
+    @mock.patch('libvirt.open', autospec=True)
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_set_secure_boot_not_uefi(self, libvirt_mock, libvirt_rw_mock):
+        with open('sushy_tools/tests/unit/emulator/domain-q35.xml', 'r') as f:
+            data = f.read()
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.assertRaises(error.NotSupportedError,
+                          self.test_driver.set_secure_boot, self.uuid, True)
